package multex; //Assertion.java

//History:
//2001-04-19  Knabe  Added parameterized check methods,
//                   AssertionFailure as outer class.
//2000-09-29  Knabe  Creation

/**Tool for checking unexpected programming errors*/
public class Assertion {

/**Do not create objects of me!*/
private Assertion(){}


/**Checks, that an assertion condition holds, detail description here.
  More conveniently you can call the convenience methods check(...) with
  0..10 additional diagnostic parameters of type Object instead of the Object[]
  here.
  <H3>Note:</H3>
  If you want to give diagnostic parameters which have to be created newly,
  e.g. to encapsulate a value of a primitive type, then you should yourself
  write the code e.g.<PRE>
*   if(!condition){
*     throw new AssertionFailure(name, stringPar1, new Integer(intPar2));
*   }
  </PRE>
  This should not be committed to a check-Method, because creating a new
  capsule object for each parameter of a primitive type involves to much
  overhead in non-error cases, too.
  @param i_condition The condition to be checked
  @param i_name The condition, which failed, as a String
  @param i_parameters A polymorphic array of additional diagnostic parameters, e.g. values of variables in the assertion condition
  @throws AssertionFailure The condition did not hold
*/
public static void check(
  final boolean i_condition, final String i_name, Object[] i_parameters
)throws AssertionFailure {
  if(!i_condition){
    throw new AssertionFailure(i_name, i_parameters);
  }
}//check

/**Convenience check method null.
  @see #check(boolean, String, Object[])
*/
public static void check(
  final boolean i_condition, final String i_name
)throws AssertionFailure {
  check(i_condition, i_name, null);
}//check

/**Convenience check method 0.
  @see #check(boolean, String, Object[])
*/
public static void check(
  final boolean i_condition, final String i_name,
  final Object i_object0
)throws AssertionFailure {
  check(i_condition, i_name, new Object[]{
    i_object0
  });
}//check

/**Convenience check method 1.
  @see #check(boolean, String, Object[])
*/
public static void check(
  final boolean i_condition, final String i_name,
  final Object i_object0, final Object i_object1
)throws AssertionFailure {
  check(i_condition, i_name, new Object[]{
    i_object0, i_object1
  });
}//check

/**Convenience check method 2.
  @see #check(boolean, String, Object[])
*/
public static void check(
  final boolean i_condition, final String i_name,
  final Object i_object0, final Object i_object1, final Object i_object2
)throws AssertionFailure {
  check(i_condition, i_name, new Object[]{
    i_object0, i_object1, i_object2
  });
}//check

/**Convenience check method 3.
  @see #check(boolean, String, Object[])
*/
public static void check(
  final boolean i_condition, final String i_name,
  final Object i_object0, final Object i_object1, final Object i_object2,
  final Object i_object3
)throws AssertionFailure {
  check(i_condition, i_name, new Object[]{
    i_object0, i_object1, i_object2, i_object3
  });
}//check

/**Convenience check method 4.
  @see #check(boolean, String, Object[])
*/
public static void check(
  final boolean i_condition, final String i_name,
  final Object i_object0, final Object i_object1, final Object i_object2,
  final Object i_object3, final Object i_object4
)throws AssertionFailure {
  check(i_condition, i_name, new Object[]{
    i_object0, i_object1, i_object2, i_object3, i_object4
  });
}//check

/**Convenience check method 5.
  @see #check(boolean, String, Object[])
*/
public static void check(
  final boolean i_condition, final String i_name,
  final Object i_object0, final Object i_object1, final Object i_object2,
  final Object i_object3, final Object i_object4, final Object i_object5
)throws AssertionFailure {
  check(i_condition, i_name, new Object[]{
    i_object0, i_object1, i_object2, i_object3, i_object4, i_object5
  });
}//check

/**Convenience check method 6.
  @see #check(boolean, String, Object[])
*/
public static void check(
  final boolean i_condition, final String i_name,
  final Object i_object0, final Object i_object1, final Object i_object2,
  final Object i_object3, final Object i_object4, final Object i_object5,
  final Object i_object6
)throws AssertionFailure {
  check(i_condition, i_name, new Object[]{
    i_object0, i_object1, i_object2, i_object3, i_object4, i_object5, i_object6
  });
}//check

/**Convenience check method 7.
  @see #check(boolean, String, Object[])
*/
public static void check(
  final boolean i_condition, final String i_name,
  final Object i_object0, final Object i_object1, final Object i_object2,
  final Object i_object3, final Object i_object4, final Object i_object5,
  final Object i_object6, final Object i_object7
)throws AssertionFailure {
  check(i_condition, i_name, new Object[]{
    i_object0, i_object1, i_object2, i_object3, i_object4, i_object5, i_object6,
    i_object7
  });
}//check

/**Convenience check method 8.
  @see #check(boolean, String, Object[])
*/
public static void check(
  final boolean i_condition, final String i_name,
  final Object i_object0, final Object i_object1, final Object i_object2,
  final Object i_object3, final Object i_object4, final Object i_object5,
  final Object i_object6, final Object i_object7, final Object i_object8
)throws AssertionFailure {
  check(i_condition, i_name, new Object[]{
    i_object0, i_object1, i_object2, i_object3, i_object4, i_object5, i_object6,
    i_object7, i_object8
  });
}//check

/**Convenience check method 9.
  @see #check(boolean, String, Object[])
*/
public static void check(
  final boolean i_condition, final String i_name,
  final Object i_object0, final Object i_object1, final Object i_object2,
  final Object i_object3, final Object i_object4, final Object i_object5,
  final Object i_object6, final Object i_object7, final Object i_object8,
  final Object i_object9
)throws AssertionFailure {
  check(i_condition, i_name, new Object[]{
    i_object0, i_object1, i_object2, i_object3, i_object4, i_object5, i_object6,
    i_object7, i_object8, i_object9
  });
}//check




}//Assertion