package multex;  //UtilTest.java

import java.io.FileNotFoundException;

import multex.test.TestUtil;

//2003-05-08  Knabe  Aus Vorlage MsgTest
//2002-05-30  Knabe  Umbau von Stack_bt gem Beispiel junit.samples.VectorTest

/**JUnit batch test driver for the class multex.Msg.
*/
public class UtilTest extends junit.framework.TestCase {


    //Testfixtures:
	private final Exc       t1  = new Exc("Kategorie nicht erlaubt");      //Leave at line 16
	private final Throwable t21 = new FileNotFoundException("kasse.dat");  //Leave at line 17
	private final Failure t2    = new Failure("Ziel nicht gefunden", t21); //Leave at line 18
	private final Exc listExc   = new Exc("Verarbeitungsfehler", t1, t2);  //Leave at line 19

	private final String className = UtilTest.class.getName();
	
	//Verwaltungsoperationen:

	public UtilTest(final String name) {
	  super(name);
	}
	
	public static void main (final String[] i_args) {
	  junit.textui.TestRunner.run(suite());
	}
	
	public static junit.framework.Test suite() {
	  final junit.framework.TestSuite result
	  = new junit.framework.TestSuite(UtilTest.class);
	  return result;
	}
	
	//Einzelne Testschritte:
	  
	/**Tests cloning an object array or null.*/
	public void testCloneObjectArray(){
		final Integer entry1 = new Integer(1);
		final Character entry2 = new Character('2');
	    final String entry3 = "333";
	    final Object[] original = {entry1, entry2, entry3};
	    final Object[] expected = {entry1, entry2, entry3};
	    TestUtil.assertEquals(expected, original);
		
		final Object[] clone = Util.clone(original);
	    TestUtil.assertEquals(expected, clone);
	    assertTrue(original!=clone);
	    for(int i=0; i<original.length; i++){
	    	original[i] = "X" + i;
	    }
	    TestUtil.assertEquals(expected, clone);
	    assertTrue(!expected[0].equals(original[0]));
	}
	
	/**Tests getting all causes (including those stored as parameters of a {@link MultexException}).*/
	public void testGetCauses(){
		final Throwable lowExc1 = new Exc("lowExc1");
		final Throwable lowExc2 = new NullPointerException("lowExc2");
		final Throwable topExc  = new Failure("topExc", lowExc1, "par0", lowExc2, "par2");
		final Throwable[] expectedCauses = new Throwable[]{lowExc1, lowExc2}; 
		final Throwable[] actualCauses = Util.getCauses(topExc);
	    TestUtil.assertEquals(expectedCauses, actualCauses);
	}
	
	/**Tests printing the stack trace for an Exc with a list of Throwables as parameters.
	*/
	public void testAppendCompactStackTrace_ListExc(){
		final StringBuffer buf = new StringBuffer();
		Util.appendCompactStackTrace(buf, listExc);
		final String result = buf.toString();
		TestUtil.assertIsContained("kasse.dat", result);
		TestUtil.assertIsContained("Ziel nicht gefunden", result);
		TestUtil.assertIsContained("Kategorie nicht erlaubt", result);
		TestUtil.assertIsContained("Verarbeitungsfehler", result); 
		final String expected =
			"+" + t1.getClass().getName() + ": " + t1.getDefaultMessageTextPattern() + Util.lineSeparator
			+ "\tat multex.UtilTest.<init>(UtilTest.java:16)" + Util.lineSeparator
			+ "++" + t21.getClass().getName() + ": " + t21.getMessage() + Util.lineSeparator
			+ "\tat multex.UtilTest.<init>(UtilTest.java:17)" + Util.lineSeparator
			+ Util.wasCausing + Util.lineSeparator
			+ "+" + t2.getClass().getName() + ": " + t2.getDefaultMessageTextPattern() + Util.lineSeparator
			+ "\tat multex.UtilTest.<init>(UtilTest.java:18)" + Util.lineSeparator
			+ Util.wasCausing + Util.lineSeparator
			+ listExc.getClass().getName() + ": " + listExc.getDefaultMessageTextPattern() + Util.lineSeparator
			+ "\tat multex.UtilTest.<init>(UtilTest.java:19)" + Util.lineSeparator
			+ "\tat sun.reflect."			
		;
		TestUtil.assertIsStart(expected, result);
	}


}//UtilTest